/******************************************************************************/
/* SOUND.C: ARM DSP Samples                                                   */
/******************************************************************************/
                  
#include <LPC213x.H>
#include "defs.h"
#include "adc.h"
#include "ff.h"

const char LEDVol[8] = { 0x01, 0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF };

int iSample = 0;
volatile int iPinPong = 0;

#define SND_BUFFER_SIDE  FF_POINTS

short SndBuffer1[SND_BUFFER_SIDE];
short SndBuffer2[SND_BUFFER_SIDE];

void SetDAC(unsigned short val)
{
	DACR = val;
}

__irq void tc0 (void)
{
#if 0
	unsigned short val;
	val = (long)ADC1ReadEnd(IN_PORT) - 0x200;
	ADC1ReadBegin(IN_PORT);

	SetDAC(((long)val + 0x200) << 3);

	if (iSample == SND_BUFFER_SIDE)
	{
		iSample = 0;
		iPinPong = iPinPong == 0 ? 1 : 0;
	}
	iSample++;
#else
	if (iSample == SND_BUFFER_SIDE)
	{
		iSample = 0;
		iPinPong = iPinPong == 0 ? 1 : 0;
	}

	if (iPinPong == 0)
	{
		SetDAC(((long)SndBuffer1[iSample] + 0x200) << 3);
		SndBuffer1[iSample] = (long)ADC1ReadEnd(IN_PORT) - 0x200;
		ADC1ReadBegin(IN_PORT);
		
		iSample++;
	}
	else
	{
		SetDAC(((long)SndBuffer2[iSample] + 0x200) << 3);
		SndBuffer2[iSample] = (long)ADC1ReadEnd(IN_PORT) - 0x200;
		ADC1ReadBegin(IN_PORT);
		
		iSample++;
	}
#endif

	T0IR        = 1;                              /* Clear interrupt flag           */
	VICVectAddr = 0;                              /* Acknowledge Interrupt          */
}

#if defined( DSP_PROC )
extern short ff_out_re_buffer[FF_POINTS];
extern short ff_out_im_buffer[FF_POINTS];

void dsp_process(unsigned short val)
{
#if DSP_MIRROR
	int i;

	//  
	for (i = 1; i < FF_POINTS/2; i++)
	{
		short re;
		short im;

		re = ff_out_re_buffer[FF_POINTS-i];
		im = ff_out_im_buffer[FF_POINTS-i];

		ff_out_re_buffer[FF_POINTS-i] = ff_out_re_buffer[i];
		ff_out_im_buffer[FF_POINTS-i] = ff_out_im_buffer[i];

		ff_out_re_buffer[i] = re;
		ff_out_im_buffer[i] = im;
	}
#endif

#if DSP_SHIFT
	int i;
	int offset;

	offset = (short)((long)val - 0x200)>>4;
	
	//  
	for (i = 1; i < FF_POINTS/2+1; i++)
	{
		short re;
		short im;

		re = ff_out_re_buffer[i];
		im = ff_out_im_buffer[i];
		
		if (i+offset < FF_POINTS/2)
		{
			ff_out_re_buffer[i+offset] = re;
			ff_out_re_buffer[i+offset] = im;
		}

		ff_out_re_buffer[i] = 0;
		ff_out_im_buffer[i] = 0;
		
		re = ff_out_re_buffer[FF_POINTS-i];
		im = ff_out_im_buffer[FF_POINTS-i];
		
		if (i+offset < FF_POINTS/2)
		{
			ff_out_re_buffer[FF_POINTS-(i+offset)] = re;
			ff_out_re_buffer[FF_POINTS-(i+offset)] = im;
		}

		ff_out_re_buffer[FF_POINTS-i] = 0;
		ff_out_im_buffer[FF_POINTS-i] = 0;
	}
#endif
}
#endif // DSP_PROC

int main (void)
{
	ADCInit(100000);
	ADC1ReadBegin(IN_PORT);
	SetDAC(0x00008000);                       /* DAC Output set to Middle Point */

	IODIR1  = 0x00FF0000;                     /* P1.16..23 defined as Outputs   */

	T0MR0 = TIM_TICK - 1;                     /* Timer0 Match Value 0           */
	T0MCR = 3;                                /* Interrupt and Reset on MR0     */
	T0TCR = 1;                                /* Timer0 Enable                  */
	T0PR  = 0;                                /* Prescale                       */
	VICVectAddr0 = (unsigned long)tc0;        /* Set Interrupt Vector in 0      */
	VICVectCntl0 = 0x20 | 4;                  /* Use it for Timer0 Interrupt    */
	VICIntEnable = 0x00000010;                /* Enable Timer0 Interrupt        */

	while (1)
	{                                         /* Loop forever                   */
		int val;
		int delay = 0;

		//  :
		val = ADC0Read(1); 

		//    :
		IOCLR1 = 0x00FF0000;                  /* Turn Off all LEDs              */
		IOSET1 = LEDVol[val >> 7] << 16;      /* LEDs show Volume Level         */

		if (iPinPong == 0)
		{
#if defined( DSP_FF )
			//     2:
			ff(SndBuffer2);
	#if defined( DSP_PROC )
			dsp_process(val);
	#endif // DSP_PROC
			ffb(SndBuffer2);
#endif // DSP_FF

			//   
			while (iPinPong == 0) delay++;
			//       -  , 
			//    .
		}
		else
		{
#if defined( DSP_FF )
			//     1:
			ff(SndBuffer1);
	#if defined( DSP_PROC )
			dsp_process(val);
	#endif // DSP_PROC
			ffb(SndBuffer1);
#endif // DSP_FF
			
			//   
			while (iPinPong == 1) delay++;
			//       -  , 
			//    .
		}

		delay = 0;

		// Check if INT1 is pressed
		if ((IOPIN0 & 0x00004000) == 0)
		{
			// ...
		}
	}
}
