#include <LPC214x.h>
#include "defs.h"
#include "adc.h"

unsigned long ADCInit( unsigned long ADC_Clk )
{
    // all the related pins are set to ADC inputs, AD0.1~4, AD1.0~7
	// and select DAC (AOUT)
    PINSEL0 = 0x0F333F00;
    PINSEL1 = 0x15501800 | 0x01080000;

    AD0CR = ( 0x01 << 0 ) | 	// SEL=1,select channel 0, 1 to 4 on ADC0
	( ( CPU_CLOCK / ADC_Clk - 1 ) << 8 ) |  // CLKDIV = Fpclk / 1000000 - 1 
	( 0 << 16 ) | 		// BURST = 0, no BURST, software controlled
	( 0 << 17 ) |  		// CLKS = 0, 11 clocks/10 bits 
	( 1 << 21 ) |  		// PDN = 1, normal operation 
	( 0 << 22 ) |  		// TEST1:0 = 00 
	( 0 << 24 ) |  		// START = 0 A/D conversion stops
	( 0 << 27 );		// EDGE = 0 (CAP/MAT singal falling,trigger A/D conversion)

    AD1CR = ( 0x01 << 0 ) | 	// SEL=1,select channel 0, 0 to 7 on ADC1
	( ( CPU_CLOCK / ADC_Clk - 1 ) << 8 ) |  // CLKDIV = Fpclk / 1000000 - 1 
	( 0 << 16 ) | 		// BURST = 0, no BURST, software controlled 
	( 0 << 17 ) |  		// CLKS = 0, 11 clocks/10 bits 
	( 1 << 21 ) |  		// PDN = 1, normal operation 
	( 0 << 22 ) |  		// TEST1:0 = 00 
	( 0 << 24 ) |  		// START = 0 A/D conversion stops
	( 0 << 27 );		// EDGE = 0 (CAP/MAT singal falling,trigger A/D conversion)

    return (TRUE);
}

unsigned long ADC0Read( unsigned char channelNum )
{
    unsigned long regVal, ADC_Data;

    /* channel number is 0 through 7 */
    if ( channelNum >= ADC_NUM )
    {
		channelNum = 0;		/* reset channel number to 0 */
    }
    AD0CR &= 0xFFFFFF00;
    AD0CR |= (1 << 24) | (1 << channelNum);	
	/* switch channel,start A/D convert */

    while ( 1 )			/* wait until end of A/D convert */
    {
		#define ADREG0_ADDR 0xE0034010
		regVal = *(volatile unsigned long *)(ADREG0_ADDR + ADC_INDEX * channelNum);
		/* read result of A/D conversion */
		if ( regVal & ADC_DONE )
		    break;
    }	
        
    AD0CR &= 0xF8FFFFFF;            /* stop ADC now */    
    if ( regVal & ADC_OVERRUN )     /* save data when it's not overrun otherwise, return zero */
    {
		return ( 0 );
    }
    ADC_Data = ( regVal >> 6 ) & 0x3FF;
    return ( ADC_Data );	/* return A/D conversion value */
}

unsigned long ADC1Read( unsigned char channelNum )
{
    unsigned long regVal, ADC_Data;

    /* channel number is 0 through 7 */
    if ( channelNum >= ADC_NUM )
    {
		channelNum = 0;		/* reset channel number to 0 */
    }
    AD1CR &= 0xFFFFFF00;
    AD1CR |= (1 << 24) | (1 << channelNum);	
				/* switch channel,start A/D convert */

    while ( 1 )			/* wait until end of A/D convert */
    {
		#define ADREG1_ADDR 0xE0060010
		regVal = *(volatile unsigned long *)(ADREG1_ADDR + ADC_INDEX * channelNum);
		/* read result of A/D conversion */
		if ( regVal & ADC_DONE )
		    break;
    }	
        
    AD1CR &= 0xF8FFFFFF;	/* stop ADC now */
    if ( regVal & ADC_OVERRUN )	/* save data when it's not overrun otherwise, return zero */
    {
		return ( 0 );
    }
    
    ADC_Data = ( regVal >> 6 ) & 0x3FF;
    return ( ADC_Data );	/* return A/D conversion value */
}

void ADC1ReadBegin( unsigned char channelNum )
{
    /* channel number is 0 through 7 */
    if ( channelNum >= ADC_NUM )
    {
		channelNum = 0;		/* reset channel number to 0 */
    }
    AD1CR &= 0xFFFFFF00;
    AD1CR |= (1 << 24) | (1 << channelNum);	
				/* switch channel,start A/D convert */
}

unsigned long ADC1ReadEnd( unsigned char channelNum )
{
    unsigned long regVal, ADC_Data;

    while ( 1 )			/* wait until end of A/D convert */
    {
		#define ADREG1_ADDR 0xE0060010
		regVal = *(volatile unsigned long *)(ADREG1_ADDR + ADC_INDEX * channelNum);
		/* read result of A/D conversion */
		if ( regVal & ADC_DONE )
		    break;
    }

    AD1CR &= 0xF8FFFFFF;	/* stop ADC now */ 
    if ( regVal & ADC_OVERRUN )	/* save data when it's not overrun otherwise, return zero */
    {
		return ( 0 );
    }
    
    ADC_Data = ( regVal >> 6 ) & 0x3FF;
    return ( ADC_Data );	/* return A/D conversion value */
}
