/*
	  FFT. 
	   FFT.
	 : , 16 ,  : 32 ,   : 16 .
	   .     2  (   "" ). 

*/

#if defined( _TEST_ )
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#endif

#include "ff.h"
#include "ff_table.c"

//short ff_buffer[FF_POINTS];

long  ff_re_buffer[FF_POINTS];
long  ff_im_buffer[FF_POINTS];

short ff_out_re_buffer[FF_POINTS];
short ff_out_im_buffer[FF_POINTS];

//extern unsigned short ff_cos_table[FF_POINTS];   //    .
//extern unsigned short ff_rev_subst[FF_POINTS]; //      (- ).

void ff(short *buffer)
{
	int layer;
	int i;

	for (i = 0; i < FF_POINTS; i++)
	{
		ff_re_buffer[i] = (long)buffer[i];
		ff_im_buffer[i] = 0;
	}

	for (layer = 0; layer < FF_ORDER; layer++)
	{
		int step = 1 << (FF_ORDER - layer - 1);
		
#if defined( _TEST_ )
		//printf("layer: %d, step = %d\n", layer, step);
#endif
		
		for (i = 0; i < FF_POINTS; )
		{
			long reA, reB;
			long imA, imB;
			int r;
			
			r = (i * (1 << layer)) & FF_MASK;
			
			// A = (a + b)
			// B = (a - b) * W
			
			reA =  ff_re_buffer[i] + ff_re_buffer[i + step];
			imA =  ff_im_buffer[i] + ff_im_buffer[i + step];
			
			reB =  ff_re_buffer[i] - ff_re_buffer[i + step];
			imB =  ff_im_buffer[i] - ff_im_buffer[i + step];
			
			//printf("A, B: (%d %d) (%d %d)\n",reA,imA,reB,imB);
			//printf("%d %d %d %d\n",reA,reB,imA,imB);
			
			ff_re_buffer[i] = reA / 2;
			ff_im_buffer[i] = imA / 2;
			
			ff_re_buffer[i+step] =  (reB * ff_cos_table[r] + imB * ff_sin_table[r]) / (1<<16); //15+1
			ff_im_buffer[i+step] =  (imB * ff_cos_table[r] - reB * ff_sin_table[r]) / (1<<16); //15+1
			
			i++;
			if (i & step)
				i += step;
		}
	}
	
#if defined( _TEST_ ) & 0
	printf("ff_re_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
	{
		printf("%d, ", ff_re_buffer[i]);
	}
	printf("};\n");
	printf("ff_im_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
	{
		printf("%d, ", ff_im_buffer[i]);
	}
	printf("};\n\n");
#endif
	
	for (i = 0; i < FF_POINTS; i++)
	{
		short re = ff_re_buffer[ff_rev_subst[i]];
		short im = ff_im_buffer[ff_rev_subst[i]];
		
		ff_out_re_buffer[i] = re;
		ff_out_im_buffer[i] = im;
		
#if defined( _TEST_ )
		buffer[i] = (short)(sqrt((long)re * re + (long)im * im));
#endif
	}
}

void ffb(short *buffer)
{
	//  
	int layer;
	int i;

	for (i = 0; i < FF_POINTS; i++)
	{
		short re = ff_out_re_buffer[i]; //ff_re_buffer[ff_rev_subst[i]];
		short im = ff_out_im_buffer[i]; //ff_im_buffer[ff_rev_subst[i]];
		
		ff_re_buffer[i] = re;
		ff_im_buffer[i] = im;
	}

	for (layer = 0; layer < FF_ORDER; layer++)
	{
		int step = 1 << (FF_ORDER - layer - 1);
		
#if defined( _TEST_ )
		//printf("layer: %d, step = %d\n", layer, step);
#endif
		
		for (i = 0; i < FF_POINTS; )
		{
			long reA, reB;
			long imA, imB;
			int r;
			
			r = (i * (1 << layer)) & FF_MASK;
			
			// A = (a + b)
			// B = (a - b) * W
			
			reA =  ff_re_buffer[i] + ff_re_buffer[i + step];
			imA =  ff_im_buffer[i] + ff_im_buffer[i + step];
			
			reB =  ff_re_buffer[i] - ff_re_buffer[i + step];
			imB =  ff_im_buffer[i] - ff_im_buffer[i + step];
			
			//printf("A, B: (%d %d) (%d %d)\n",reA,imA,reB,imB);
			//printf("%d %d %d %d\n",reA,reB,imA,imB);
			
			ff_re_buffer[i] = reA;
			ff_im_buffer[i] = imA;
			
			ff_re_buffer[i+step] =  (reB * ff_cos_table[r] - imB * ff_sin_table[r]) / (1<<15);
			ff_im_buffer[i+step] =  (imB * ff_cos_table[r] + reB * ff_sin_table[r]) / (1<<15);
			
			i++;
			if (i & step)
				i += step;
		}
	}
	
	for (i = 0; i < FF_POINTS; i++)
	{
		short re = ff_re_buffer[ff_rev_subst[i]];
		short im = ff_im_buffer[ff_rev_subst[i]];
		
		ff_out_re_buffer[i] = re;
		ff_out_im_buffer[i] = im;
	}
	
	for (i = 0; i < FF_POINTS; i++)
	{
		buffer[i] = (short)ff_re_buffer[ff_rev_subst[i]];
		//ff_im_buffer[i] = 0;
	}

}

#if defined( _TEST_ )

#define PI 3.14159265358979

void main(void)
{
	int i;
	
	for (i=0; i < FF_POINTS; i++)
	{
		//ff_buffer[i] = (short)(sin((double)i * 2 * PI * 10 / FF_POINTS) * ((1<<15) - 1));
		//ff_buffer[i] = (short)(sin((double)i * 2 * PI * 3 / FF_POINTS) * ((1<<14) - 1)) + (1<<14);
		ff_buffer[i] = (short)(cos((double)i * 2 * PI * 3 / FF_POINTS) * ((1<<13) - 1));
	}
	
	printf("ff_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ff_buffer[i]);
	printf("};\n\n");

#if 0
	for (i = 0; i < FF_POINTS; i++)
	{
		int j;
		printf(" %3d |", i);
		for (j = 0; j < ((ff_buffer[i] + (1<<15)) / 1000); j++)
		{
			printf(".");
		}
		printf("\n");
	}
#endif

	ff(ff_buffer);

	printf("ff_re_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ff_out_re_buffer[i]);
	printf("};\n");
	printf("ff_im_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ff_out_im_buffer[i]);
	printf("};\n\n");
	
	printf("ff_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ff_buffer[i]);
	printf("};\n\n");

#if 0
	for (i = 0; i < FF_POINTS; i++)
	{
		int j;
		printf(" %3d |", i);
		for (j = 0; j < ((ff_buffer[i] + (1<<15)) / 1000); j++)
		{
			printf(".");
		}
		printf("\n");
	}
#endif

	//      
	for (i = 0; i < FF_POINTS; i++)
	{
		ff_re_buffer[i]  =  ff_out_re_buffer[i];
		ff_im_buffer[i]  =  ff_out_im_buffer[i];
	}
	
	ffb(ff_buffer);
	
	printf("ff_re_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ff_out_re_buffer[i]);
	printf("};\n");
	printf("ff_im_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ff_out_im_buffer[i]);
	printf("};\n\n");

	printf("ff_buffer[] = { ");
	for (i=0; i < FF_POINTS; i++)
		printf("%d, ", ff_buffer[i]);
	printf("};\n\n");

#if 0
	for (i = 0; i < FF_POINTS; i++)
	{
		int j;
		printf(" %3d |", i);
		for (j = 0; j < ((ff_buffer[i] + (1<<15)) / 1000); j++)
		{
			printf(".");
		}
		printf("\n");
	} 
#endif

}
#endif // _TEST_
